# Generated by Django 3.1.14 on 2022-04-12 13:13

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('fees', '0003_auto_20220412_1512'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='packagequota',
            unique_together={('package', 'quota')},
        ),
    ]
