# Generated by Django 3.1.14 on 2022-04-04 19:28

from django.db import migrations, models
import modeltrans.fields


class Migration(migrations.Migration):

    dependencies = [
        ('fees', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='package',
            name='description',
            field=models.TextField(blank=True, verbose_name='description'),
        ),
        migrations.AddField(
            model_name='package',
            name='i18n',
            field=modeltrans.fields.TranslationField(fields=('description',), required_languages=(), virtual_fields=True),
        ),
        migrations.AlterField(
            model_name='package',
            name='is_visible',
            field=models.BooleanField(db_index=True, default=True, help_text='Is visible in pricing page', verbose_name='visible'),
        ),
        migrations.AlterField(
            model_name='package',
            name='title',
            field=models.CharField(max_length=50, unique=True, verbose_name='title'),
        ),
    ]
