# Generated by Django 3.1.14 on 2023-02-22 23:08

from django.db import migrations, models
import modeltrans.fields


class Migration(migrations.Migration):

    dependencies = [
        ('fees', '0009_auto_20220706_1719'),
    ]

    operations = [
        migrations.AlterField(
            model_name='package',
            name='i18n',
            field=modeltrans.fields.TranslationField(fields=('title', 'description'), required_languages=(), virtual_fields=True),
        ),
        migrations.AlterField(
            model_name='plan',
            name='expiration',
            field=models.DateField(blank=True, db_index=True, default=None, help_text='keep blank to calculate automatically by pricing period', null=True, verbose_name='expires'),
        ),
        migrations.AlterUniqueTogether(
            name='pricing',
            unique_together={('package', 'period', 'duration')},
        ),
    ]
