{% load i18n %}
<strong>{% trans 'Plan' %}:</strong> {{ object.package }}<br>
{% if object.pricing %}
    <strong>{% trans 'Pricing' %}:</strong> {{ object.pricing }}<br>
{% endif %}
{% if object.expiration %}
    <strong>{% trans 'Expires' %}:</strong> {{ object.expiration }} ({% trans 'in' %} {{ object.days_left }} {% trans 'days' %})<br>
{% endif %}
<a href="{% url 'fees:pricing' %}" class="btn btn-primary">{% trans 'Pricing' %}</a>