# Generated by Django 3.1.14 on 2022-07-06 15:07

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('fees', '0007_auto_20220706_1652'),
    ]

    operations = [
        migrations.AddField(
            model_name='plan',
            name='activation',
            field=models.DateField(auto_now_add=True, default=django.utils.timezone.now, verbose_name='activation'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='package',
            name='is_default',
            field=models.BooleanField(db_index=True, default=False, help_text='Default package for new purchaser (useful for trial packages). Only 1 default package at a time can be set.', verbose_name='default'),
        ),
        migrations.AlterField(
            model_name='package',
            name='is_fallback',
            field=models.BooleanField(db_index=True, default=False, help_text='Fallback package for purchaser when its subscription expires or trial ends (useful for freemium packages). Only 1 fallback package at a time can be set.', verbose_name='fallback'),
        ),
    ]
