# Generated by Django 3.1.14 on 2022-07-06 15:19

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('fees', '0008_auto_20220706_1707'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='plan',
            options={'get_latest_by': 'expiration', 'ordering': ['-activation'], 'verbose_name': 'Plan', 'verbose_name_plural': 'Plans'},
        ),
        migrations.AlterField(
            model_name='plan',
            name='purchaser',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='provisioning.workspace', verbose_name='purchaser'),
        ),
    ]
