# Generated by Django 3.1.14 on 2022-07-06 14:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('fees', '0005_auto_20220706_1625'),
    ]

    operations = [
        migrations.AddConstraint(
            model_name='package',
            constraint=models.UniqueConstraint(condition=models.Q(is_fallback=True), fields=('is_fallback',), name='unique_fallback'),
        ),
    ]
