{% load i18n %}

<strong>{{ package }}</strong><br>
{% for package_quota in package.packagequota_set.all %}
    <u>{{ package_quota.quota.name_i18n }}</u>:{{ package_quota.value }} {{ package_quota.quota.unit }}
    ({{ package_quota.quota.description_i18n }})<br>
{% endfor %}

{% for pricing in package.pricing_set.all %}
    {{ pricing.get_price_display }}<br>
    {{ pricing.get_duration_display }}<br>
    {% if pricing.price_per_month != pricing.price %}
        (~{{ pricing.get_price_per_month_display }} / {% trans 'month' %})<br>
    {% endif %}
    <a href="{{ pricing.get_add_to_cart_url }}" class="d-block btn btn-outline-dark rounded-0 {% if not pricing.package.is_available %}disabled{% endif %}">{% trans 'Choose' %}</a><br>
    <br>
{% empty %}
    <a href="#" class="d-block btn btn-outline-dark rounded-0 {% if not pricing.package.is_available %}disabled{% endif %}">{% trans 'Choose' %}</a><br>
    <br>
{% endfor %}
